﻿namespace RadSchedulerApp.DatabaseProvider
{
	using Data;
	using System;
	using System.Collections.Generic;
	using System.Data.Entity.Migrations;
	using System.Linq;
	using System.Web;
	using Telerik.Web.UI;

	public class EfSchedulerProvider : SchedulerProviderBase
	{
		private RadSchedulerDbContext db = new RadSchedulerDbContext();

		public EfSchedulerProvider()
			: base()
		{
		}

		public override IEnumerable<Appointment> GetAppointments(ISchedulerInfo schedulerInfo)
		{
			var dbAppointments = this.db.Appointments.ToList();

			var appointmentsToReturn = new List<Appointment>();
			foreach (var dbAppointment in dbAppointments)
			{
				var appointment = new Appointment()
				{
					ID = dbAppointment.ID,
					Subject = dbAppointment.Subject,
					Start = dbAppointment.Start,
					Description = dbAppointment.Description,
					End = dbAppointment.End,
					RecurrenceRule = dbAppointment.RecurrenceRule,
					RecurrenceParentID = dbAppointment.RecurrenceParentID
				};

				if (appointment.RecurrenceParentID != null)
				{
					appointment.RecurrenceState = RecurrenceState.Exception;
				}
				else
				{
					if (appointment.RecurrenceRule != string.Empty)
					{
						appointment.RecurrenceState = RecurrenceState.Master;
					}
				}

				appointmentsToReturn.Add(appointment);
			}

			return appointmentsToReturn;
		}

		public override void Insert(ISchedulerInfo schedulerInfo, Appointment appointmentToInsert)
		{
			var newAppointment = new Models.Appointment
			{
				Subject = appointmentToInsert.Subject,
				Start = appointmentToInsert.Start,
				Description = appointmentToInsert.Description,
				End = appointmentToInsert.End,
				RecurrenceRule = appointmentToInsert.RecurrenceRule,
				RecurrenceParentID = (int?)appointmentToInsert.RecurrenceParentID
			};

			this.db.Appointments.AddOrUpdate(newAppointment);
			this.db.SaveChanges();
		}

		public override void Update(ISchedulerInfo schedulerInfo, Appointment appointmentToUpdate)
		{
			var dbAppointment = new Models.Appointment
			{
				ID = (int)appointmentToUpdate.ID,
				Subject = appointmentToUpdate.Subject,
				Start = appointmentToUpdate.Start,
				Description = appointmentToUpdate.Description,
				End = appointmentToUpdate.End,
				RecurrenceRule = appointmentToUpdate.RecurrenceRule,
				RecurrenceParentID = (int?)appointmentToUpdate.RecurrenceParentID
				
			};

			this.db.Appointments.AddOrUpdate(this.db.Appointments.FirstOrDefault(a => a.ID == (int)appointmentToUpdate.ID), dbAppointment);
			this.db.SaveChanges();
		}

		public override void Delete(ISchedulerInfo schedulerInfo, Appointment appointmentToDelete)
		{
			var dbAppointment = this.db.Appointments.FirstOrDefault(a => a.ID == (int)appointmentToDelete.ID);

			this.db.Appointments.Remove(dbAppointment);
			this.db.SaveChanges();
		}

		public override IEnumerable<ResourceType> GetResourceTypes(RadScheduler owner)
		{
			var resoursesList = new List<ResourceType>();
			return resoursesList;
		}

		public override IEnumerable<Resource> GetResourcesByType(RadScheduler owner, string resourceType)
		{
			var resourcesList = new List<Resource>();
			return resourcesList;
		}
	}
}